@@ -2,17 +2,16 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2020 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
-   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
-   27th April 2017).
+   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
+   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
 
-   End User License Agreement: www.juce.com/juce-5-licence
-   Privacy Policy: www.juce.com/juce-5-privacy-policy
+   End User License Agreement: www.juce.com/juce-6-licence
+   Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
    www.gnu.org/licenses).
@@ -39,7 +38,7 @@ struct Oversampling<SampleType>::OversamplingStage
     virtual ~OversamplingStage() {}
 
     //==============================================================================
-    virtual SampleType getLatencyInSamples() = 0;
+    virtual SampleType getLatencyInSamples() const = 0;
 
     virtual void initProcessing (size_t maximumNumberOfSamplesBeforeOversampling)
     {
@@ -53,13 +52,13 @@ struct Oversampling<SampleType>::OversamplingStage
         buffer.clear();
     }
 
-    dsp::AudioBlock<SampleType> getProcessedSamples (size_t numSamples)
+    AudioBlock<SampleType> getProcessedSamples (size_t numSamples)
     {
-        return dsp::AudioBlock<SampleType> (buffer).getSubBlock (0, numSamples);
+        return AudioBlock<SampleType> (buffer).getSubBlock (0, numSamples);
     }
 
-    virtual void processSamplesUp (dsp::AudioBlock<SampleType>&) = 0;
-    virtual void processSamplesDown (dsp::AudioBlock<SampleType>&) = 0;
+    virtual void processSamplesUp   (const AudioBlock<const SampleType>&) = 0;
+    virtual void processSamplesDown (AudioBlock<SampleType>&) = 0;
 
     AudioBuffer<SampleType> buffer;
     size_t numChannels, factor;
@@ -78,12 +77,12 @@ struct OversamplingDummy   : public Oversampling<SampleType>::OversamplingStage
     OversamplingDummy (size_t numChans) : ParentType (numChans, 1) {}
 
     //==============================================================================
-    SampleType getLatencyInSamples() override
+    SampleType getLatencyInSamples() const override
     {
         return 0;
     }
 
-    void processSamplesUp (dsp::AudioBlock<SampleType>& inputBlock) override
+    void processSamplesUp (const AudioBlock<const SampleType>& inputBlock) override
     {
         jassert (inputBlock.getNumChannels() <= static_cast<size_t> (ParentType::buffer.getNumChannels()));
         jassert (inputBlock.getNumSamples() * ParentType::factor <= static_cast<size_t> (ParentType::buffer.getNumSamples()));
@@ -93,7 +92,7 @@ struct OversamplingDummy   : public Oversampling<SampleType>::OversamplingStage
                 inputBlock.getChannelPointer (channel), static_cast<int> (inputBlock.getNumSamples()));
     }
 
-    void processSamplesDown (dsp::AudioBlock<SampleType>& outputBlock) override
+    void processSamplesDown (AudioBlock<SampleType>& outputBlock) override
     {
         jassert (outputBlock.getNumChannels() <= static_cast<size_t> (ParentType::buffer.getNumChannels()));
         jassert (outputBlock.getNumSamples() * ParentType::factor <= static_cast<size_t> (ParentType::buffer.getNumSamples()));
@@ -122,8 +121,8 @@ struct Oversampling2TimesEquirippleFIR  : public Oversampling<SampleType>::Overs
                                      SampleType stopbandAmplitudedBDown)
         : ParentType (numChans, 2)
     {
-        coefficientsUp   = *dsp::FilterDesign<SampleType>::designFIRLowpassHalfBandEquirippleMethod (normalisedTransitionWidthUp,   stopbandAmplitudedBUp);
-        coefficientsDown = *dsp::FilterDesign<SampleType>::designFIRLowpassHalfBandEquirippleMethod (normalisedTransitionWidthDown, stopbandAmplitudedBDown);
+        coefficientsUp   = *FilterDesign<SampleType>::designFIRLowpassHalfBandEquirippleMethod (normalisedTransitionWidthUp,   stopbandAmplitudedBUp);
+        coefficientsDown = *FilterDesign<SampleType>::designFIRLowpassHalfBandEquirippleMethod (normalisedTransitionWidthDown, stopbandAmplitudedBDown);
 
         auto N = coefficientsUp.getFilterOrder() + 1;
         stateUp.setSize (static_cast<int> (this->numChannels), static_cast<int> (N));
@@ -139,7 +138,7 @@ struct Oversampling2TimesEquirippleFIR  : public Oversampling<SampleType>::Overs
     }
 
     //==============================================================================
-    SampleType getLatencyInSamples() override
+    SampleType getLatencyInSamples() const override
     {
         return static_cast<SampleType> (coefficientsUp.getFilterOrder() + coefficientsDown.getFilterOrder()) * 0.5f;
     }
@@ -155,7 +154,7 @@ struct Oversampling2TimesEquirippleFIR  : public Oversampling<SampleType>::Overs
         position.fill (0);
     }
 
-    void processSamplesUp (dsp::AudioBlock<SampleType>& inputBlock) override
+    void processSamplesUp (const AudioBlock<const SampleType>& inputBlock) override
     {
         jassert (inputBlock.getNumChannels() <= static_cast<size_t> (ParentType::buffer.getNumChannels()));
         jassert (inputBlock.getNumSamples() * ParentType::factor <= static_cast<size_t> (ParentType::buffer.getNumSamples()));
@@ -195,7 +194,7 @@ struct Oversampling2TimesEquirippleFIR  : public Oversampling<SampleType>::Overs
         }
     }
 
-    void processSamplesDown (dsp::AudioBlock<SampleType>& outputBlock) override
+    void processSamplesDown (AudioBlock<SampleType>& outputBlock) override
     {
         jassert (outputBlock.getNumChannels() <= static_cast<size_t> (ParentType::buffer.getNumChannels()));
         jassert (outputBlock.getNumSamples() * ParentType::factor <= static_cast<size_t> (ParentType::buffer.getNumSamples()));
@@ -248,7 +247,7 @@ struct Oversampling2TimesEquirippleFIR  : public Oversampling<SampleType>::Overs
 
 private:
     //==============================================================================
-    dsp::FIR::Coefficients<SampleType> coefficientsUp, coefficientsDown;
+    FIR::Coefficients<SampleType> coefficientsUp, coefficientsDown;
     AudioBuffer<SampleType> stateUp, stateDown, stateDown2;
     Array<size_t> position;
 
@@ -274,11 +273,11 @@ struct Oversampling2TimesPolyphaseIIR  : public Oversampling<SampleType>::Oversa
                                     SampleType stopbandAmplitudedBDown)
         : ParentType (numChans, 2)
     {
-        auto structureUp = dsp::FilterDesign<SampleType>::designIIRLowpassHalfBandPolyphaseAllpassMethod (normalisedTransitionWidthUp, stopbandAmplitudedBUp);
+        auto structureUp = FilterDesign<SampleType>::designIIRLowpassHalfBandPolyphaseAllpassMethod (normalisedTransitionWidthUp, stopbandAmplitudedBUp);
         auto coeffsUp = getCoefficients (structureUp);
         latency = static_cast<SampleType> (-(coeffsUp.getPhaseForFrequency (0.0001, 1.0)) / (0.0001 * MathConstants<double>::twoPi));
 
-        auto structureDown = dsp::FilterDesign<SampleType>::designIIRLowpassHalfBandPolyphaseAllpassMethod (normalisedTransitionWidthDown, stopbandAmplitudedBDown);
+        auto structureDown = FilterDesign<SampleType>::designIIRLowpassHalfBandPolyphaseAllpassMethod (normalisedTransitionWidthDown, stopbandAmplitudedBDown);
         auto coeffsDown = getCoefficients (structureDown);
         latency += static_cast<SampleType> (-(coeffsDown.getPhaseForFrequency (0.0001, 1.0)) / (0.0001 * MathConstants<double>::twoPi));
 
@@ -300,7 +299,7 @@ struct Oversampling2TimesPolyphaseIIR  : public Oversampling<SampleType>::Oversa
     }
 
     //==============================================================================
-    SampleType getLatencyInSamples() override
+    SampleType getLatencyInSamples() const override
     {
         return latency;
     }
@@ -313,7 +312,7 @@ struct Oversampling2TimesPolyphaseIIR  : public Oversampling<SampleType>::Oversa
         delayDown.fill (0);
     }
 
-    void processSamplesUp (dsp::AudioBlock<SampleType>& inputBlock) override
+    void processSamplesUp (const AudioBlock<const SampleType>& inputBlock) override
     {
         jassert (inputBlock.getNumChannels() <= static_cast<size_t> (ParentType::buffer.getNumChannels()));
         jassert (inputBlock.getNumSamples() * ParentType::factor <= static_cast<size_t> (ParentType::buffer.getNumSamples()));
@@ -364,11 +363,12 @@ struct Oversampling2TimesPolyphaseIIR  : public Oversampling<SampleType>::Oversa
             }
         }
 
-        // Snap To Zero
+       #if JUCE_DSP_ENABLE_SNAP_TO_ZERO
         snapToZero (true);
+       #endif
     }
 
-    void processSamplesDown (dsp::AudioBlock<SampleType>& outputBlock) override
+    void processSamplesDown (AudioBlock<SampleType>& outputBlock) override
     {
         jassert (outputBlock.getNumChannels() <= static_cast<size_t> (ParentType::buffer.getNumChannels()));
         jassert (outputBlock.getNumSamples() * ParentType::factor <= static_cast<size_t> (ParentType::buffer.getNumSamples()));
@@ -422,8 +422,9 @@ struct Oversampling2TimesPolyphaseIIR  : public Oversampling<SampleType>::Oversa
             delayDown.setUnchecked (static_cast<int> (channel), delay);
         }
 
-        // Snap To Zero
+       #if JUCE_DSP_ENABLE_SNAP_TO_ZERO
         snapToZero (false);
+       #endif
     }
 
     void snapToZero (bool snapUpProcessing)
@@ -457,12 +458,12 @@ private:
     /** This function calculates the equivalent high order IIR filter of a given
         polyphase cascaded allpass filters structure.
     */
-    dsp::IIR::Coefficients<SampleType> getCoefficients (typename dsp::FilterDesign<SampleType>::IIRPolyphaseAllpassStructure& structure) const
+    IIR::Coefficients<SampleType> getCoefficients (typename FilterDesign<SampleType>::IIRPolyphaseAllpassStructure& structure) const
     {
         constexpr auto one = static_cast<SampleType> (1.0);
 
-        dsp::Polynomial<SampleType> numerator1 ({ one }), denominator1 ({ one }),
-                                    numerator2 ({ one }), denominator2 ({ one });
+        Polynomial<SampleType> numerator1 ({ one }), denominator1 ({ one }),
+                               numerator2 ({ one }), denominator2 ({ one });
 
         for (auto* i : structure.directPath)
         {
@@ -470,13 +471,13 @@ private:
 
             if (i->getFilterOrder() == 1)
             {
-                numerator1 = numerator1.getProductWith (dsp::Polynomial<SampleType> ({ coeffs[0], coeffs[1] }));
-                denominator1 = denominator1.getProductWith (dsp::Polynomial<SampleType> ({ one, coeffs[2] }));
+                numerator1   = numerator1  .getProductWith (Polynomial<SampleType> ({ coeffs[0], coeffs[1] }));
+                denominator1 = denominator1.getProductWith (Polynomial<SampleType> ({ one,       coeffs[2] }));
             }
             else
             {
-                numerator1 = numerator1.getProductWith (dsp::Polynomial<SampleType> ({ coeffs[0], coeffs[1], coeffs[2] }));
-                denominator1 = denominator1.getProductWith (dsp::Polynomial<SampleType> ({ one, coeffs[3], coeffs[4] }));
+                numerator1   = numerator1  .getProductWith (Polynomial<SampleType> ({ coeffs[0], coeffs[1], coeffs[2] }));
+                denominator1 = denominator1.getProductWith (Polynomial<SampleType> ({ one,       coeffs[3], coeffs[4] }));
             }
         }
 
@@ -486,13 +487,13 @@ private:
 
             if (i->getFilterOrder() == 1)
             {
-                numerator2 = numerator2.getProductWith (dsp::Polynomial<SampleType> ({ coeffs[0], coeffs[1] }));
-                denominator2 = denominator2.getProductWith (dsp::Polynomial<SampleType> ({ one, coeffs[2] }));
+                numerator2   = numerator2  .getProductWith (Polynomial<SampleType> ({ coeffs[0], coeffs[1] }));
+                denominator2 = denominator2.getProductWith (Polynomial<SampleType> ({ one,       coeffs[2] }));
             }
             else
             {
-                numerator2 = numerator2.getProductWith (dsp::Polynomial<SampleType> ({ coeffs[0], coeffs[1], coeffs[2] }));
-                denominator2 = denominator2.getProductWith (dsp::Polynomial<SampleType> ({ one, coeffs[3], coeffs[4] }));
+                numerator2   = numerator2  .getProductWith (Polynomial<SampleType> ({ coeffs[0], coeffs[1], coeffs[2] }));
+                denominator2 = denominator2.getProductWith (Polynomial<SampleType> ({ one,       coeffs[3], coeffs[4] }));
             }
         }
 
@@ -501,15 +502,15 @@ private:
         auto numerator   = numeratorf1.getSumWith (numeratorf2);
         auto denominator = denominator1.getProductWith (denominator2);
 
-        dsp::IIR::Coefficients<SampleType> coeffs;
+        IIR::Coefficients<SampleType> coeffs;
 
         coeffs.coefficients.clear();
         auto inversion = one / denominator[0];
 
-        for (auto i = 0; i <= numerator.getOrder(); ++i)
+        for (int i = 0; i <= numerator.getOrder(); ++i)
             coeffs.coefficients.add (numerator[i] * inversion);
 
-        for (auto i = 1; i <= denominator.getOrder(); ++i)
+        for (int i = 1; i <= denominator.getOrder(); ++i)
             coeffs.coefficients.add (denominator[i] * inversion);
 
         return coeffs;
@@ -539,8 +540,9 @@ Oversampling<SampleType>::Oversampling (size_t newNumChannels)
 
 template <typename SampleType>
 Oversampling<SampleType>::Oversampling (size_t newNumChannels, size_t newFactor,
-                                        FilterType newType, bool isMaximumQuality)
-    : numChannels (newNumChannels)
+                                        FilterType newType, bool isMaximumQuality,
+                                        bool useIntegerLatency)
+    : numChannels (newNumChannels), shouldUseIntegerLatency (useIntegerLatency)
 {
     jassert (isPositiveAndBelow (newFactor, 5) && numChannels > 0);
 
@@ -561,8 +563,8 @@ Oversampling<SampleType>::Oversampling (size_t newNumChannels, size_t newFactor,
             auto gaindBFactorDown = (isMaximumQuality ? 10.0f  : 8.0f);
 
             addOversamplingStage (FilterType::filterHalfBandPolyphaseIIR,
-                                  twUp, gaindBStartUp + gaindBFactorUp * n,
-                                  twDown, gaindBStartDown + gaindBFactorDown * n);
+                                  twUp, gaindBStartUp + gaindBFactorUp * (float) n,
+                                  twDown, gaindBStartDown + gaindBFactorDown * (float) n);
         }
     }
     else if (newType == FilterType::filterHalfBandFIREquiripple)
@@ -578,8 +580,8 @@ Oversampling<SampleType>::Oversampling (size_t newNumChannels, size_t newFactor,
             auto gaindBFactorDown = (isMaximumQuality ? 10.0f  : 8.0f);
 
             addOversamplingStage (FilterType::filterHalfBandFIREquiripple,
-                                  twUp, gaindBStartUp + gaindBFactorUp * n,
-                                  twDown, gaindBStartDown + gaindBFactorDown * n);
+                                  twUp, gaindBStartUp + gaindBFactorUp * (float) n,
+                                  twDown, gaindBStartDown + gaindBFactorDown * (float) n);
         }
     }
 }
@@ -629,7 +631,20 @@ void Oversampling<SampleType>::clearOversamplingStages()
 
 //==============================================================================
 template <typename SampleType>
-SampleType Oversampling<SampleType>::getLatencyInSamples() noexcept
+void Oversampling<SampleType>::setUsingIntegerLatency (bool useIntegerLatency) noexcept
+{
+    shouldUseIntegerLatency = useIntegerLatency;
+}
+
+template <typename SampleType>
+SampleType Oversampling<SampleType>::getLatencyInSamples() const noexcept
+{
+    auto latency = getUncompensatedLatency();
+    return shouldUseIntegerLatency ? latency + fractionalDelay : latency;
+}
+
+template <typename SampleType>
+SampleType Oversampling<SampleType>::getUncompensatedLatency() const noexcept
 {
     auto latency = static_cast<SampleType> (0);
     size_t order = 1;
@@ -644,7 +659,7 @@ SampleType Oversampling<SampleType>::getLatencyInSamples() noexcept
 }
 
 template <typename SampleType>
-size_t Oversampling<SampleType>::getOversamplingFactor() noexcept
+size_t Oversampling<SampleType>::getOversamplingFactor() const noexcept
 {
     return factorOversampling;
 }
@@ -662,6 +677,10 @@ void Oversampling<SampleType>::initProcessing (size_t maximumNumberOfSamplesBefo
         currentNumSamples *= stage->factor;
     }
 
+    ProcessSpec spec = { 0.0, (uint32) maximumNumberOfSamplesBeforeOversampling, (uint32) numChannels };
+    delay.prepare (spec);
+    updateDelayLine();
+
     isReady = true;
     reset();
 }
@@ -674,29 +693,33 @@ void Oversampling<SampleType>::reset() noexcept
     if (isReady)
         for (auto* stage : stages)
            stage->reset();
+
+    delay.reset();
 }
 
 template <typename SampleType>
-typename dsp::AudioBlock<SampleType> Oversampling<SampleType>::processSamplesUp (const dsp::AudioBlock<SampleType>& inputBlock) noexcept
+AudioBlock<SampleType> Oversampling<SampleType>::processSamplesUp (const AudioBlock<const SampleType>& inputBlock) noexcept
 {
     jassert (! stages.isEmpty());
 
     if (! isReady)
         return {};
 
-    auto audioBlock = inputBlock;
+    auto* firstStage = stages.getUnchecked (0);
+    firstStage->processSamplesUp (inputBlock);
+    auto block = firstStage->getProcessedSamples (inputBlock.getNumSamples() * firstStage->factor);
 
-    for (auto* stage : stages)
+    for (int i = 1; i < stages.size(); ++i)
     {
-        stage->processSamplesUp (audioBlock);
-        audioBlock = stage->getProcessedSamples (audioBlock.getNumSamples() * stage->factor);
+        stages[i]->processSamplesUp (block);
+        block = stages[i]->getProcessedSamples (block.getNumSamples() * stages[i]->factor);
     }
 
-    return audioBlock;
+    return block;
 }
 
 template <typename SampleType>
-void Oversampling<SampleType>::processSamplesDown (dsp::AudioBlock<SampleType>& outputBlock) noexcept
+void Oversampling<SampleType>::processSamplesDown (AudioBlock<SampleType>& outputBlock) noexcept
 {
     jassert (! stages.isEmpty());
 
@@ -718,6 +741,26 @@ void Oversampling<SampleType>::processSamplesDown (dsp::AudioBlock<SampleType>&
     }
 
     stages.getFirst()->processSamplesDown (outputBlock);
+
+    if (shouldUseIntegerLatency && fractionalDelay > static_cast<SampleType> (0.0))
+    {
+        auto context = ProcessContextReplacing<SampleType> (outputBlock);
+        delay.process (context);
+    }
+}
+
+template <typename SampleType>
+void Oversampling<SampleType>::updateDelayLine()
+{
+    auto latency = getUncompensatedLatency();
+    fractionalDelay = static_cast<SampleType> (1.0) - (latency - std::floor (latency));
+
+    if (fractionalDelay == static_cast<SampleType> (1.0))
+        fractionalDelay = static_cast<SampleType> (0.0);
+    else if (fractionalDelay < static_cast<SampleType> (0.618))
+        fractionalDelay += static_cast<SampleType> (1.0);
+
+    delay.setDelay (fractionalDelay);
 }
 
 template class Oversampling<float>;
